#include "Event.h"
#include "EventList.h"
#include "Py.h"

int Packet::live_packets = 0;

message & 
message::operator , (Event * e)
{
	fprintf (f, "e=(%.3f, 0x%03x, (%02x, %02x), %.3f)", 
			e->t, e->type, e->data[0], e->data[1], e->v);
	return *this;
}

message & 
message::operator , (Packet * p)
{
	const char * s = p->payload ? ", ..." : "";
	byte * data = p->payload ? (byte *) p->payload : p->data;

	fprintf (f, "p=(%.3f, 0x%03x, (%02x, %02x%s), %.3f)", 
			p->t, p->type, data[0], data[1], s, p->v);

	return *this;
}

float
Event::compare (Event & e)
{
	return (float) t - e.t;
}

/* ///////////////////////////////////////////////////// Event publics */

PyObject *
Event::as_object()
{
	return Py_BuildValue ("di(ii)d", t, type, data[0], data[1], v);
}

void
Event::parse (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {"t", "type", "data", "v", 0};
	
	int tp;
	PyObject * d = 0;
	int b[2] = {0, 0};

	if (!PyArg_ParseTupleAndKeywords (args, kwds, "di|Od", keys, 
				&t, &tp, &d, &v))
		throw blewit();

	if (tp & ~(Event::Mask))
		throw ValueError() + "type: " + tp;
	type = (Event::Type) tp;

	if (d)
	{
		if (!PyArg_ParseTuple (d, "i|i", b, b+1))
			throw ValueError() + "'data' is at most two (MIDI) bytes";
		
		/* quietly saturate to MIDI data range */
		for (int i = 0; i < 2; ++i)
		{
			if (b[i] < 0) b[i] = 0;
			else if (b[i] > 127) b[i] = 127; 
		}
	}

	data[0] = b[0];
	data[1] = b[1];
}

/* ////////////////////////////////////// Packet publics */

PyObject *
Packet::as_object()
{
	char * d;
	int n;
	
	if (payload) d = (char *) payload, n = length;
	else d = (char *) data, n = 2;

	return Py_BuildValue ("dis#d", t, type, d, n, v);
}

bool
Packet::parse (PyObject * args)
{
	int tp;
	void * d;
	int n;

	if (!PyArg_ParseTuple (args, "dis#|d", &t, &tp, &d, &n, &v))
		return false;

	if (tp & ~(Packet::Mask))
		throw ValueError() + "type: " + tp;
	type = (Packet::Type) tp;

	set_data ((byte *) d, n);

	return true;
}

/* /////////////////////////////////////////////////////////// */

EventList::EventList()
{
	plugin.flags = Cycle::Tick;
	new_tx<TX> ("tx");
}

EventList::~EventList()
{
}

void
EventList::cycle (Cycle & c)
{
	Lock _lock (mutex);

	int i = _index (c.tick.then);
	while (i < events)
	{
		if (events[i].t >= c.tick.now)
			break;

		Packet * p = c.packet();
		(Event &) *p = events[i];
		c.push (p, tx(0));
		
		++i;
	}
}

/* /////////////////////////////////////////////////////////// */

void
EventList::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {0};
	
	if (!PyArg_ParseTupleAndKeywords (args, kwds, "", keys))
		throw RuntimeError();
}

PyObject *
EventList::sq_item (int i)
{
	if (i < 0 || i >= events)
		throw IndexError (i);

	return events[i].as_object();
}

PyObject *
EventList::index (PyObject * args)
{
	Time::tick t;

	if (!PyArg_ParseTuple (args, "d", &t))
		return 0;

	int i = _index (t);

	return PyInt (i);
}

PyObject *
EventList::add (PyObject * args, PyObject * kwds)
{
	Event e;

	e.parse (args, kwds);

	Lock _lock (mutex);
	
	events.insert (e, _index (e.t));

	return PyNone();
}

#include "Event.thunk.h"
#if PYTHUNK
	Class (EventList, Plugin)
	Method (EventList :: add, KW)
	Method (EventList :: index, VA)
	sq_length (EventList)
	sq_item (EventList)
#endif

void 
init_Event (PyObject * dict)
{
	new PyType<EventList> ("EventList", dict);
}
