#ifndef _EVENT_H_
#define _EVENT_H_

#include "basics.h"
#include "List.h"
#include "Mutex.h"

class Event
{
	public:
		/* type values */
		typedef enum {
			Invalid = 0,

			NoteOff = 0x8,
			NoteOn = 0x9, /* v = length */
			AfterTouch = 0xA,
			CC = 0xB,
			PrgChange = 0xC,
			ChanPress = 0xD,
			Pitch = 0xE,
			
			/* 0x80 - 0xFF are reserved for MIDI messages */

			Tempo = 0x100, /* v = bpm */

			Audio = 0x200,

			Mask = 0x0FFF
		} Type;

	public:
		double t;

		Type type: 12;
		int selected: 1;
					
		byte data[2];
		float v;

		float compare (Event & e);

		bool operator < (Event & e)
			{ return compare (e) < 0; }
		bool operator > (Event & e)
			{ return compare (e) > 0; }
				
		void set_type (int tp)
			{ type = (Event::Type) tp; }

		/* /////////////////////////////// */
		
		PyObject * as_object();
		void parse (PyObject * args, PyObject * kwds);
};

class Packet
: public Event
{
	public:
		static int live_packets;

		void * payload;
		int length;

		Packet()
			{
				length = 0;
				payload = 0;
			}

		template <class T>
		void set_data (T * t, int n)
			{
				if (n <= 2 && sizeof (T) == 1)
				{
					data[0] = (byte) t[0];
					data[1] = (byte) t[1];
				}
				else
				{
					length = n;
					payload = malloc (n * sizeof (T));
					if (payload == 0) throw MemoryError();
					memcpy (payload, t, n * sizeof (T));
					++live_packets;
				}
			}

		void free() 
			{ 
				if (payload == 0)
					return;
				if (type == Audio)
				{
					payload = 0;
					return;
				}

				--live_packets;
				::free (payload); 
				payload = 0;
			}
		
		void set_audio (sample * s, int n)
			{
				type = Audio;
				payload = s;
				length = n;
			}

		PyObject * as_object();
		bool parse (PyObject * args);
};

#endif /* _EVENT_H_ */
