#ifndef _FIFO_H_
#define _FIFO_H_

template <class T, int N>
class FIFO
{
	private:
		T * stack;
		int r, w, used; 

	public:
		FIFO()
			{
				reset();
				stack = (T *) calloc (N, sizeof (T));
				if (!stack) throw MemoryError();
			}
		
		~FIFO()
			{
				if (stack) ::free (stack);
			}

		void reset() { used = r = w = 0; }

		int size() { return N; }
		int have() { return used; }
		int free() { return N - used; }

		void push (T t)
			{
				if (used == N) 
					throw OverflowError() + "FIFO full";

				stack[w] = t;
				w = (w + 1) % N;
				++used;
			}

		T pop()
			{
				if (used == 0) throw EOFError() + "FIFO empty";
				
				T t = stack[r];
				r = (r + 1) % N;
				--used;

				return t;
			}

		T peek()
			{
				if (used == 0) return 0;
				return stack[r];
			}
};

#endif /* _FIFO_H_ */
