#ifndef _FLOW_H_
#define _FLOW_H_

#include "Thread.h"
#include "RTC.h"
#include "Tempo.h"
#include "Plugin.h"

class AudioIO;

class Request
{
	public:
		char * name;
		blewit blown;
		
		Request (char * n) { name = n; }
		virtual ~Request() { }

		virtual void exec (Flow *) { msg(), "void request"; }
};

class MasterCycle
{
	public:
		Cycle time, tick, audio;
		IOCycle time_io, tick_io;

		FIFO<Packet *, 8 * PACKET_FIFO_SIZE> recycled; /* intermittent storage in recycle */
			
		MasterCycle (Flow * f, Tempo * t) 
			: time (f, Cycle::Time, PACKET_FIFO_SIZE), 
				tick (f, Cycle::Tick, PACKET_FIFO_SIZE),
				audio (f, Cycle::Tick, PACKET_FIFO_SIZE),
				time_io (f, PACKET_FIFO_SIZE),
				tick_io (f, PACKET_FIFO_SIZE)
			{ 
				time.tempo = t;
				tick.tempo = t;
				audio.tempo = t;
			}

		void recycle();
		void recycle_from (Recycler & r);
		void recycle_to (Recycler & r);
};

class Flow
: public Object
{
	public:
		Mutex mutex;

		MasterCycle cycle;

		RTC rtc;
		ThreadT<Flow> rt_events;

		AudioIO * audio_io;

		ObjList<Plugin, 32> plugins;
		FIFO<Request *, 128> requests;

	public:
		void handle_thread_event (Thread * t, int fd);
		void handle_io (IOCycle & c, int fd);

		ThreadT<Flow> thread;

		void handle_trigger (Thread * t, void * trigger);

		/* script plugins that have rx packets are passed via this pipe */
		Pipe run_script;
		void script_cycle();

		/* request answers written to this pipe */
		Pipe rq_answer;
		void process_requests();
		void request (Request * r); /* outside */
		void handle_request (Request * r); /* inside */

		void set_audio_io (AudioIO *);

	public:
		Flow();
		~Flow();

		struct {
			double last;
		} pulse;

		void time_cycle();
		void tick_cycle();
		void audio_cycle (int frames);

		void post_cycle(); /* check for script plugins that received during cycle */

		void decycle();

		void seek_to_time (Time::time);

		void seek_to_tick (Time::tick t)
			{ seek_to_time (tempo.map_tick (t)); }

		Tempo tempo;

		void add_plugin (Py<Plugin> *);
		void rm_plugin (Py<Plugin> *);

		/* //////////////////////////////////////////////////////// */

		void __init__ (PyObject * args, PyObject * kwds);

		int set_time (PyObject * args);
		PyObject * get_time();

		int set_tick (PyObject * args);
		PyObject * get_tick();

		int set_audio_io (PyObject * args);
		PyObject * get_audio_io();

		PyObject * _insert (PyObject * args);
		PyObject * _connect (PyObject * args);

		PyObject * _push (PyObject * args);

		int sq_length();
		PyObject * sq_item (int i);

		int _traverse (visitproc visit, void * arg);
		int _clear();
};

#endif /* _FLOW_H_ */
