#ifndef _JACK_H_
#define _JACK_H_

#include "Audio.h"
#include <jack/jack.h>

class JackPort
{
	public:
		jack_port_t * port;

		JackPort() { port = 0; }
		
		void setup (jack_client_t * client, char * name, JackPortFlags flags)
			{
				port = jack_port_register (client, name, 
						JACK_DEFAULT_AUDIO_TYPE, flags, 0);
				if (port == 0)
					throw IOError() + "cannot register port " + name;
			}

		sample * get_samples (jack_nframes_t frames)
			{
				return (sample *) jack_port_get_buffer (port, frames);
			}
};

class JackIn
: public TX, public JackPort
{
	public:
		void setup (jack_client_t * client, char * name)
			{
				JackPort::setup (client, name, JackPortIsInput);
			}
};

class JackOut
: public RX, public JackPort
{
	public:
		void setup (jack_client_t * client, char * name)
			{
				JackPort::setup (client, name, JackPortIsOutput);
			}
};

class Jack
: public AudioIO
{
	private:
		jack_client_t * client;
		bool jackd_is_down, first_run;
		
	public:
		Jack();
		~Jack();

		virtual void set_flow (Flow *);

		int jack_buffer_size (jack_nframes_t n)
			{
				buffer_size = n;
				update_flow();
				return 0;
			}

		int jack_sample_rate (jack_nframes_t n)
			{
				sample_rate = n;
				update_flow();
				return 0;
			}

		void jack_shutdown();

		int jack_process (int frames);
		void cycle_tx (int frames);
		void cycle_rx (int frames);

		void update_flow();

		/* ////////////////////////////////////////////////// */

		void __init__ (PyObject * args, PyObject * kwds);

		PyObject * _connect (PyObject * args);
};

#endif /* _JACK_H_ */
