#ifndef _LIST_H_
#define _LIST_H_

#include "basics.h"

template <class T, int G>
class _List
{
	protected:
		int allocated;
		
	public:
		T * items;
		int length;
		
		enum {
			Start = 0, 
			End = -1
		};

		_List()
			{
				length = 0;
				init();
			}
		
		~_List()
			{
				if (items)
					free (items);
			}

		operator T * ()
			{ return items; }

		operator int () 
			{ return length; }

		int index (T t)
			{
				for (int i = 0; i < length; ++i)
					if (items[i] == t)
						return i;
				throw LookupError() + t;
			}

	protected:
		void init()
			{
				allocated = G;
				items = (T *) malloc (allocated * sizeof (T));
				if (items == 0)
					throw MemoryError() + "allocating " + allocated + " items";
			}

		void reallocate (int n)
			{
				int a = n + (G - (n % G));

				if (a == allocated)
					return;

				allocated = a;
				
				items	= (T *) realloc (items, sizeof (T) * allocated);
			}

		/* returns the index where room for 'n' items has been made */
		int make_room (int n, int i = End)
			{
				reallocate (length + n);

				if (i >= 0 && i < length)
				{
					/* need to move items */
					length += n;
					memmove	(items + i + n, items + i, (length - i) * sizeof (T));
				}
				else
				{
					i = length;
					length += n;
				}

				return i;
			}

	public:
		void remove_items (int i, int n)
			{
				int end = i + n;

				if (i < 0 || i >= length)
					throw IndexError (i);
				if (end < 0 || end > length)
					throw IndexError (end);

				if (length - end)
					memmove (items + i, items + end, (length - end) * sizeof (T));

				reallocate (length = length - n);
			}

};

template <class T, int G>
class List
: public _List<T, G>
{
	public:
		const char * name;
		
		enum {
			Start = 0, 
			End = -1
		};

		/* returns the insertion index */
		int insert (T & t, int i)
			{
				i = make_room (1, i);
				items[i] = t;
				return i;
			}

		int append()
			{ return make_room (1); }
		
		int append (T & t)
			{ return insert (t, End); }
		
		T & operator [] (int i) 
			{ return items[i]; }

		T * operator + (int i)
			{ return items + i; }
};

#endif /* _LIST_H_ */
