#ifndef _OBJ_LIST_H_
#define _OBJ_LIST_H_

#include "List.h"
#include "Mutex.h"
#include "Py.h"

template <class O, int G>
class ObjList
: public Object, public _List<Py<O> *, G>
{
	public:
		Mutex mutex;

		void append (Py<O> * o)
			{ 
				incref (o);
				items [make_room (1)] = o;
			}
		
		void remove (Py<O> * o)
			{
				int i = index (o);
				decref (o);
				remove_items (i, 1);
			}
		
		O * operator [] (int i)
			{ return &items[i]->obj; }
		
		~ObjList()
			{ _clear(); }

		/* /////////////////////////////////////////////////////////// */

		void __init__ (PyObject * args, PyObject * kwds)
			{ throw RuntimeError() + "You cannot instantiate this"; }
		
		int sq_length()
			{ return length; }
		
		PyObject * sq_item (int i)
			{ 
				if (i < 0 || i >= length)
					throw IndexError (i);
				return incref (items[i]); 
			}

		int _traverse (visitproc visit, void * arg)
			{ 
				for (int i = 0; i < length; ++i)
					VISIT (items[i]);
				return 0;
			}

		int _clear()
			{
				for (int i = 0; i < length; ++i)
					decref (items[i]);
				length = 0;
				return 0;
			}
};

#endif /* _OBJ_LIST_H_ */
