#include "basics.h"
#include "Plugin.h"
#include "Metronome.h"
#include "Flow.h"
#include "PyLock.h"

Plugin::Plugin()
{
	plugin.flags = 0;
	plugin.fd = -1;

	plugin._rx = new Py<RXList>();
	plugin._tx = new Py<TXList>();
}

Plugin::~Plugin()
{
	decref (plugin._rx);
	decref (plugin._tx);
}

PyObject *
Plugin::_new_tx (PyObject * args)
{
	char * name;

	if (!PyArg_ParseTuple (args, "s", &name))
		return 0;

	TX * t = new_tx<TX> (name);
	t->flags |= TX::Script;
	return incref (t->as_py_object());
}

PyObject *
Plugin::_new_rx (PyObject * args)
{
	char * name;

	if (!PyArg_ParseTuple (args, "s", &name))
		return 0;

	RX * r = new_rx<RX> (name);
	r->flags |= RX::Script;
	plugin.flags |= Cycle::Script;
	return incref (r->as_py_object());
}

/* /////////////////////////////////// */

void
Plugin::disconnect()
{
	for (int i = 0; i < plugin._tx->obj; ++i)
		tx(i)->disconnect();
}

/* //////////////////////////////////////////// */

int
Plugin::rx_packets()
{
	int n = 0;

	for (int i = 0; i < plugin._rx->obj; ++i)
		n += rx(i)->fifo.have();

	return n;
}

Packet *
Plugin::next_packet (Time::tick t, int * r)
{
	Packet def, * packet = &def;
	packet->t = t;
	*r = -1;

	for (int i = 0; i < plugin._rx->obj; ++i)
	{
		Packet * p = rx(i)->fifo.peek();	
		if (p == 0 || *p > *packet)
			continue;
		packet = p;
		*r = i;
	}

	if (*r < 0)
		return 0;
	
	return rx(*r)->fifo.pop();
}

/* /////////////////////////////////////////////////////////////////// */

void
Plugin::cycle (Cycle & c)
{
	if (c.id != Cycle::Tick)
		throw AssertionError() + "You cannot run this realtime";
	
	Py<Object> * self = (Py<Object> *) this->as_py_object();
	Py<Object> * flow = (Py<Object> *) c.flow->as_py_object();
	xdecref (self->call ("cycle", "Off", flow, c.tick.then, c.tick.now));
}

void
Plugin::event (Cycle & c)
{
	if (c.id != Cycle::Tick)
		throw AssertionError() + "You cannot run this realtime";
	
	Py<Object> * self = (Py<Object> *) this->as_py_object();
	Py<Object> * flow = (Py<Object> *) c.flow->as_py_object();

	PyObject * o = Py_None;
	while (o)
	{
		int r;
		Packet * p = next_packet ((Time::time) 100000000000000., &r);
		if (p == 0)
			break;
		if (p->type == Event::Audio)
			continue;
		o = self->call ("event", "OOi", flow, p->as_object(), r);
		xdecref (o);
	}
}

/* ///////////////////////////////////////////// */

void
Plugin::handle_io (int fd, IOCycle & c)
{
	TRAP;
	throw AssertionError() + as_py_object() + " cannot handle IO";
}

void
Plugin::__init__ (PyObject * args, PyObject * kwds)
{
	static char * keys [] = {"flags", 0};
	plugin.flags = Cycle::Tick;

	if (!PyArg_ParseTupleAndKeywords (args, kwds, "|i", keys, &plugin.flags))
		throw blewit();
}

/* /////////////////////////////////////////////////////////////////// */

void 
Metronome::cycle (Cycle & c)
{
	Packet * p;

	p = c.packet (&event);
	p->t = c.tick.then;
	c.push (p, tx(0));

	if (event.type == Event::NoteOn)
	{
		p = c.packet (&event);
		p->t = c.tick.now - c.quarter * .1;
		p->data[1] = 0;
		c.push (p, tx(0));
	}
}

void 
Metronome::__init__ (PyObject * args, PyObject * kwds)
{
	plugin.flags = Cycle::Tick;
	new_tx<TX> ("click");
}

/* /////////////////////////////////////////////////////////////////// */

#include "Plugin.thunk.h"

#if PYTHUNK
	Class (Plugin, Object)
	GetSet (Plugin :: flags)
	Get (Plugin :: rx)
	Get (Plugin :: tx)
	Method (Plugin :: _new_rx, VA)
	Method (Plugin :: _new_tx, VA)

	Class (Metronome, Plugin)
	GetSet (Metronome :: event)
#endif

void
init_Plugin (PyObject * dict)
{
	new PyType<Plugin> ("Plugin", dict);
	new PyType<Metronome> ("Metronome", dict);
}
