#ifndef _PLUGIN_H_
#define _PLUGIN_H_

#include "Event.h"
#include "RXTX.h"
#include "ObjList.h"
#include "Cycle.h"

class Plugin
: public Object
{
	public:
		struct {
			int flags;
			int fd;

			Py<RXList> * _rx;
			Py<TXList> * _tx;

		} plugin;

		Plugin();
		~Plugin();

		virtual void cycle (Cycle &);
		virtual void event (Cycle &);
		virtual void handle_io (int fd, IOCycle &);

		void disconnect();

		RX * rx (int i)
			{
				return plugin._rx->obj[i];
			}
		TX * tx (int i)
			{
				return plugin._tx->obj[i];
			}

		template <class R>
		R * new_rx (char * name)
			{
				Py<R> * o = new Py<R>();

				o->obj.set_name (name);

				plugin._rx->obj.append ((Py<RX> *) o);
				decref (o);
				return &(o->obj);
			}

		template <class T>
		T * new_tx (char * name)
			{
				Py<T> * o = new Py<T>();

				o->obj.set_name (name);

				plugin._tx->obj.append ((Py<TX> *) o);
				decref (o);
				return &(o->obj);
			}

		/* number of packets waiting to be serviced */
		int rx_packets();

		/* pops the next packet and sets *r to rx index */
		Packet * next_packet (Time::tick t, int * r);

		/* /////////////////////////////////////////////////////////////////// */

		void __init__ (PyObject * args, PyObject * kwds);

		PyObject * get_rx() { return incref (plugin._rx); }
		PyObject * get_tx() { return incref (plugin._tx); }

		int set_flags (PyObject * args)
			{ return !PyArg_Parse (args, "i", &plugin.flags); }
		PyObject * get_flags() { return PyInt (plugin.flags); }

		PyObject * _new_rx (PyObject * args);
		PyObject * _new_tx (PyObject * args);
};

#endif /* _PLUGIN_H_ */
