#ifndef _PY_H_
#define _PY_H_

template <class O>
class Py
: public PyObject
{
	public:
		O obj;

		static PyTypeObject * type;

		void * operator new (size_t size)
			{ return (void *) PyType_GenericAlloc (type, 1); }

		PyObject * call (char * method, char * fmt, ...);
};

template <class O>
class PyType
: public PyTypeObject
{
	public:
		void * operator new (size_t size)
			{
				return PyObject_MALLOC (size);
			}

		PyType (char * name, PyObject * dict)
			{
				Py<O>::type = this;

				memset (this, 0, sizeof (*this));
				
				ob_refcnt = 1;
				ob_size = sizeof (*this);
				ob_type = &PyType_Type;
				tp_basicsize = sizeof (Py<O>);
				tp_name = name;
				
				tp_doc = "Write me a letter";
				
				tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_GC;

				tp_getattro = PyObject_GenericGetAttr;
				tp_setattro = PyObject_GenericSetAttr;
						
				tp_new = (newfunc) _new;
				tp_init = (initproc) _init;

				tp_dealloc = (destructor) _dealloc;
				tp_free = (freefunc) _free;

				tp_traverse = (traverseproc) _traverse;
				tp_clear = (inquiry) _clear;

				/* fit the type with methods, getsets and inheritance information. */
				__init__ (dict);

				PyType_Ready (this);
				PyDict_SetItemString (dict, tp_name, (PyObject *) this);
			}

		void __init__ (PyObject * dict);

		static void * _new (PyTypeObject * type, PyObject * args, PyObject * kwds)
			{
				Py<O> * self = (Py<O> *) PyType_GenericAlloc (type, 1);
				try 
				{
					/* this operator new does nothing, but the default constructor will
					 * be called. */
					new (&self->obj) O();
					return self;
				} 
				catch (blewit & b) 
				{
					PyObject_GC_Del (self);
					b.raise();
				}
				return 0;
			}

		static int _init (Py<O> * self, PyObject * args, PyObject * kwds)
			{
				try 
				{
					self->obj.__init__ (args, kwds);
					return 0;
				}
				catch (blewit & b)
				{
					b.raise();
				}
				return -1;
			}

		static void _dealloc (Py<O> * self)
			{
				try 
				{
					self->obj.O::~O();
				}
				catch (blewit & b)
				{
					msg(), "~", self, " blew it: ", b.s, ", ignoring";
				}
				PyObject_GC_UnTrack (self);
			}

		static void _free (Py<O> * self)
			{
				PyObject_GC_Del (self);
			}

		static int _traverse (Py<O> * self, visitproc visit, void * arg)
			{	return self->obj._traverse (visit, arg); }

		static int _clear (Py<O> * self)
			{	return self->obj._clear(); }
};

/* convenience constructors ////////////////////////////////////////////// */

inline PyObject *
PyNone()
{
	return ::incref (Py_None);
}

inline PyObject * 
PyInt (long l)
{
	return PyInt_FromLong (l);
}

inline PyObject * 
PyFloat (double d)
{
	return PyFloat_FromDouble (d);
}

inline PyObject * 
PyString (char * s)
{
	return PyString_FromString (s);
}

#endif /* _PY_H_ */
