#include "basics.h"
#include "Flow.h"
#include "Audio.h"
#include "Py.h"

void
Flow::__init__ (PyObject * args, PyObject * kwds)
{
	tempo.recalculate (0, 0);
	thread.start (this, "Flow");
	rt_events.start (this, "RT Events");
	rtc.start (this);
}

/* //////////////////////////////////////////////// Requests */

class RqInsert
: public Request
{
	public:
		Py<Plugin> * plugin;
		
		RqInsert() : Request ("insert") { }

		void exec (Flow * f)
			{ 
				f->add_plugin (plugin); 
			}
};

PyObject * 
Flow::_insert (PyObject * args)
{
	RqInsert r;
	
	if (!PyArg_ParseTuple (args, "O!", Py<Plugin>::type, &r.plugin))
		return 0;

	request (&r);

	return PyNone();
}

/* /////////////////////////////////////// */

class RqConnect
: public Request
{
	public:
		TX * tx;
		RX * rx;
		
		RqConnect() : Request ("connect") { }

		void exec (Flow * f)
			{ 
				tx->disconnect();
				tx->connect (rx);
			}
};

PyObject * 
Flow::_connect (PyObject * args)
{
	Py<TX> * tx; 
	Py<RX> * rx;
	
	if (!PyArg_ParseTuple (args, "O!O!", 
				Py<TX>::type, &tx, Py<RX>::type, &rx))
		return 0;

	RqConnect r;
	r.tx = &tx->obj;
	r.rx = &rx->obj;
	request (&r);

	return PyNone();
}

/* //////////////////////////////////////////// Event transmission */

PyObject *
Flow::_push (PyObject * args)
{
	PyObject * _p, * _x;

	if (!PyArg_ParseTuple (args, "OO", &_p, &_x))
		return 0;

	if (_x->ob_type == Py<TX>::type)
	{
		TX & tx = ((Py<TX> *) _x)->obj;
		if (!((tx.flags & TX::Script) || tx.rx == 0))
			throw RuntimeError() + _x + " is in use";

		Packet * p = cycle.tick.packet (_p);
		cycle.tick.push (p, &tx);
	}
	else if (_x->ob_type == Py<RX>::type)
	{
		RX & rx = ((Py<RX> *) _x)->obj;
		if (rx.tx)
			throw RuntimeError() + _x + " is in use";
		
		if (!rx.fifo.free())
			throw OverflowError() + _x + " is full";

		Packet * p = cycle.tick.packet (_p);
		rx.fifo.push (p);
	}
	else
	{
		throw TypeError() + "needs a TX or RX, not " + _x->ob_type->tp_name;
	}
	return PyNone();
}

/* //////////////////////////////////////////////////// Python publics */

int
Flow::set_time (PyObject * args)
{
	Time::time t;
	
	if (!PyArg_Parse (args, "d", &t))
		return -1;
	
	seek_to_time (t);
	return 0;
}

PyObject *
Flow::get_time()
{
	return PyFloat (cycle.time.time.now);
}

int
Flow::set_tick (PyObject * args)
{
	Time::tick t;
	
	if (!PyArg_Parse (args, "d", &t))
		return -1;
	
	seek_to_tick (t);
	return 0;
}

PyObject *
Flow::get_tick()
{
	return PyFloat (cycle.time.tick.now);
}

/* ////////////////////////////////// */

int
Flow::sq_length()
{
	return plugins.sq_length();
}

PyObject *
Flow::sq_item (int i)
{
	return plugins.sq_item (i);
}

/* /////////////////////////////////////////// */

int 
Flow::_traverse (visitproc visit, void * arg)
{
	if (audio_io)
		VISIT (audio_io->as_py_object());
	return 0;
}

int 
Flow::_clear()
{
	decref (audio_io->as_py_object());
	audio_io = 0;
	return 0;
}

/* ////////////////////////////////////////////////////// Type information */

#include "PyFlow.thunk.h"

#if PYTHUNK
	Class (Flow, Object)
	
	GetSet (Flow :: time)
	GetSet (Flow :: tick)

	GetSet (Flow :: audio_io)
	
	Method (Flow :: _insert, VA)
	Method (Flow :: _connect, VA)
	Method (Flow :: _push, VA)

	sq_length (Flow)
	sq_item (Flow)
#endif

void 
init_Flow (PyObject * dict)
{
	new PyType<Flow> ("Flow", dict);
}
