#include "basics.h"
#include <sys/ioctl.h>
#include <linux/rtc.h>
#include "Thread.h"
#include "Flow.h"

RTC::RTC()
{
	fd = -1;
}

void
RTC::start (Flow * f)
{
	flow = f;

	if (!start_rtc())
		msg(), "/dev/rtc unavailable";
	else
		thread.pfds.add_fd (fd, POLLIN);

	msg(), "RTC::fd = ", fd;
	thread.interval = 10;
	thread.start (this, "RTC");
}

bool
RTC::start_rtc()
{
	fd = open ("/dev/rtc", O_RDONLY);

	if (fd < 0)
		return 0;
		
	int f = 1 << 10;
	while (f > 1)
	{
		if (ioctl (fd, RTC_IRQP_SET, f) >= 0)
		{
			msg(), "rtc f=", f, " Hz";
			break;
		}
		f >>= 1;
	}

	return ioctl (fd, RTC_PIE_ON) >= 0;
}

void
RTC::handle_thread_event (Thread * t, int _fd)
{
	if (_fd == fd && fd >= 0)
	{
		_fd = -1;

		/* read 4 bytes for this interrupt */
		int r;
		read (fd, &r, sizeof (int));
	}

	if (_fd < 0)
		flow->time_cycle();
	else
		flow->handle_thread_event (t, _fd);
}

void
RTC::handle_trigger (Thread * t, void * trig)
{
	flow->handle_trigger (t, trig);
}
