#ifndef _RXTX_H_
#define _RXTX_H_

#include "FIFO.h"
#include "ObjList.h"

class RX;
class TX;
class Cycle;

class _X
: public Object
{
	public:
		char * name;

		enum {
			Script = 1
		};
		int flags;

		_X()
			{ flags = 0; name = 0; }
		
		void set_name (char * n)
			{	name = strdup (n); }

		~_X()
			{ if (name) free (name); }

		/* //////////////////////////////// */

		void __init__ (PyObject * args, PyObject * kwds)
			{ throw NotImplementedError() + "You cannot instantiate this"; }

		PyObject * get_name()
			{ return PyString (name); }
};

/* /////////////////////////////////////////////////////////////////// */

class RX
: public _X
{
	public:
		TX * tx;
		FIFO<Packet *, 128> fifo;

		RX() { tx = 0; }

		bool push (Packet * e)
			{
				if (this == 0)
					return false;
				fifo.push (e);
				return true;
			}

		inline void connect (TX * t);
		inline void disconnect();

		Packet * get_audio (Cycle *);
					
		/* //////////////////////////////////////////// */

		;
};

class TX
: public _X
{
	public:
		int flags;
		RX * rx;

		TX () { rx = 0; }

		bool push (Packet * e)
			{ return rx->push (e); }
		
		void disconnect()
			{ 
				msg(), "disconnect: ", name;
				if (this == 0 || rx == 0)
					return;
				rx->tx = 0;
				rx = 0;
			}

		void connect (RX * r)
			{ (rx = r)->tx = this; }

		/* ///////////////////////////////////////////// */

		;
};

inline void
RX::connect (TX * t)
{ 
	t->connect (this);
}

inline void
RX::disconnect()
{ 
	tx->disconnect();
}

/* /////////////////////////////////////////////////////// */

class RXList
: public ObjList<RX, 8>
{
	public:
		;
};

class TXList
: public ObjList<TX, 8>
{
	public:
		;
};

/* //////////////////////////////// */

class AudioRX
: public RX
{
	public:
};

/* /////////// */

class AudioTX
: public TX
{
	public:
		sample * buffer;
		int size;

		AudioTX()
			{ buffer = 0; size = 0; }
		
		void set_size (int s)
			{
				if (size == s) return;
				if (buffer) free (buffer);
				buffer = (sample *) calloc (size = s, sizeof (sample));
			}
};

#endif /* _RXTX_H_ */
