#ifndef _RECYCLER_H_
#define _RECYCLER_H_

#include "Event.h"
#include "FIFO.h"

#define PACKET_FIFO_SIZE	1024

class PacketFIFO
: public FIFO<Packet *, PACKET_FIFO_SIZE>
{ };

class Recycler
{
	public:
		friend class Flow;
		friend class MasterCycle;

	private: 
		Packet * memory;
	private:
		int N;
		PacketFIFO cache;
		PacketFIFO trash;

	public:
		Recycler (int n)
			{
				if (n > PACKET_FIFO_SIZE) throw AssertionError();
				N = n;
				memory = (Packet *) calloc (n, sizeof (Packet));
				if (memory == 0) throw MemoryError();
				refill();
			}

		void refill()
			{
				while (trash.have())
					trash.pop()->free();
				
				cache.reset();
				for (int i = 0; i < N; ++i)
					cache.push (memory + i);
			}

		~Recycler()
			{
				if (memory) free (memory);
			}

		Packet * pop()
			{ return cache.pop(); }

		void push (Packet * p)
			{ trash.push (p); }
};

#endif /* _RECYCLER_H_ */
