#ifndef _TEMPO_H_
#define _TEMPO_H_

#include "Event.h"
#include "List.h"

class TempoChange
{
	public:
		Time::time time;
		Time::tick tick;
		Time::time time_per_tick;

		TempoChange()
			{
				time = tick = 0;
				/* 60 s/m, 96 ticks/q, 96 q/m */
				time_per_tick = 60. / 96. / 96.;
			}

		Time::time 
			map_tick (Time::tick t)
			{
				t -= tick;
				t *= time_per_tick;
				return time + t;
			}

		Time::tick 
			map_time (Time::time t)
			{
				t -= time;
				t /= time_per_tick;
				return tick + t;
			}
};

class Tempo
{
	public:
		List<TempoChange, 2000> changes;

		Tempo() { }

		void recalculate (Event * e, int n)
			{
				TempoChange c;
				changes.length = 0;
				changes.append (c);
			}

		Time::time map_tick (Time::tick t)
			{
				return changes [by_tick (t)].map_tick (t);
			}

		Time::time map_time (Time::time t)
			{
				return changes [by_time (t)].map_time (t);
			}

		int by_time (Time::time t)
			{
				int hi = (int) changes - 1, lo = -1;

				while (hi - lo > 1)
				{
					int probe = (hi + lo) / 2;

					if (changes[probe].time < t)
						lo = probe;
					else
						hi = probe;
				}

				return hi;
			}

		int by_tick (Time::tick t)
			{
				int hi = (int) changes - 1, lo = -1;

				while (hi - lo > 1)
				{
					int probe = (hi + lo) / 2;

					if (changes[probe].time < t)
						lo = probe;
					else
						hi = probe;
				}

				return hi;
			}
};

#endif /* _TEMPO_H_ */
