#ifndef _basics_h_
#define _basics_h_

#include <Python.h>
#include <sys/types.h>
#include <pthread.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

typedef __uint8_t			byte;
typedef float sample;

typedef __int8_t			int8;
typedef __uint8_t			uint8;
typedef __int16_t			int16;
typedef __uint16_t		uint16;
typedef __int32_t			int32;
typedef __uint32_t		uint32;
typedef __int64_t			int64;
typedef __uint64_t		uint64;

template <class T, class Q>
T min (T t, Q q)
{
	return t < q ? t : q;
}

template <class T, class Q>
T max (T t, Q q)
{
	return t > q ? t : q;
}

#if defined (__i386__) && defined (DEBUG)
	#define TRAP asm ("int $3;")
#else
	#define TRAP
#endif

/* ///////////////////////////////////////////////////////// */

namespace Time {
	typedef double time;
	typedef double tick;
	typedef int64 audio;
}

static inline Time::time
now()
{
	static struct timeval tv;
	static long tv_sec = 0;

  gettimeofday (&tv, 0);

	/* program start is zero, 
	 * so resolution increases in the float time type */
	if (tv_sec == 0)
		tv_sec = tv.tv_sec;
	
	return (Time::time) (tv.tv_sec - tv_sec) + 
			((Time::time) tv.tv_usec) / 1000000.;
}

#include "msg.h"
#include "exceptions.h"
#include "Object.h"

class StopWatch
{
	public:
		char * action;
		Time::time then;

		StopWatch (char * a) 
			{
				action = a;
				then = now();
			}

		~StopWatch()
			{
				message(), action, ": ", now() - then, "s";
			}
};

#endif /* _basics_h_ */
