#ifndef _exceptions_h_
#define _exceptions_h_

#include <exception>

class blewit
: public std::exception
{
	public:
		PyObject * excuse;
		char s[400];

		blewit()
			{
				excuse = 0;
				s[0] = 0;
			}
		
		virtual const char * what() { return s; }

		void raise()
				{
					if (excuse && !PyErr_Occurred())
						PyErr_SetString (excuse, s);
				}

		blewit & operator + (int i)
				{	
					sprintf (s + strlen (s), "%d", i); 
					return *this;
				}

		blewit & operator + (const char * m)
				{ 
					strcat (s, m); 
					return *this;
				}

		blewit & operator + (PyObject * o)
			{
				if (o == 0) strcat (s, "<PyObject at 0>");
				else sprintf (s + strlen (s), "<%s at %p>", o->ob_type->tp_name, o);
				return *this;
			}
};

class MemoryError
: public blewit
{
	public:
		MemoryError()
			{ excuse = PyExc_MemoryError; }
};

class RuntimeError
: public blewit
{
	public:
		RuntimeError()
			{ excuse = PyExc_RuntimeError; }
};

class NotImplementedError
: public blewit
{
	public:
		NotImplementedError()
			{ excuse = PyExc_NotImplementedError; }
};

class TypeError
: public blewit
{
	public:
		TypeError()
			{ excuse = PyExc_TypeError; }
};

class IndexError
: public blewit
{
	public:
		IndexError (int i)
			{ 
				excuse = PyExc_IndexError; 
				*this + i;
			}
};

class LookupError
: public blewit
{
	public:
		LookupError()
			{ excuse = PyExc_LookupError; }
};

class ValueError
: public blewit
{
	public:
		ValueError()
			{ excuse = PyExc_ValueError; }
};

class OverflowError
: public blewit
{
	public:
		OverflowError()
			{ excuse = PyExc_OverflowError; }
};

class EOFError
: public blewit
{
	public:
		EOFError()
			{ excuse = PyExc_EOFError; }
};

class OSError
: public blewit
{
	public:
		OSError (int e = 0)
			{ 
				excuse = PyExc_OSError;
				e = e ? e : errno;
				sprintf (s, "[%d] %s", e, strerror (e));
			}
};

class IOError
: public OSError
{
	public:
		IOError (int e = 0)
			: OSError (e)
			{ excuse = PyExc_IOError; }
};

class _AssertionError
: public blewit
{
	public:
		_AssertionError (char * f, int l)
			{
				excuse = PyExc_AssertionError;
				(*this) + f + ":" + l + " ";
				message(), s;
				TRAP;
			}
};

#define AssertionError() _AssertionError (__FILE__, __LINE__)

#endif /* _exceptions_h_ */
