#include "basics.h"
#include "Py.h"
#include "Thread.h"
#include "Event.h"
#include <dlfcn.h>

extern void init_Event (PyObject * dict);
extern void init_Flow (PyObject * dict);
extern void init_RXTX (PyObject * dict);
extern void init_Plugin (PyObject * dict);
extern void init_MIDI (PyObject * dict);

extern void init_ALSA (PyObject * dict);
extern void init_Jack (PyObject * dict);

void finimilk();

int Object::live_objects = 0;
pthread_key_t Thread::key = 0;

/* //////////////////////////////////////////// */

FILE * message::f = 0;
PyObject * message::file = 0;

void
message::init (char * path)
{
	f = fopen (path, "w");
	if (f == 0) f = stderr;
	file = PyFile_FromFile (f, "stdmsg", "w", 0);
}

void
message::fini()
{
	decref (file);
	if (f != stderr) fclose (f);
}

message::message()
{
	Thread * t = Thread::current();
	if (t) *this, "[", t->name, "] ";
}

message &
message::py_err()
{
	PyObject * type, * value, * trace;
	PyErr_Fetch (&type, &value, &trace);
	
	if (type)
	{
		PyObject * s = PyObject_Str (type);
		*this, PyString_AsString (s), ": ";
		decref (s);
	}

	if (value)
	{
		PyObject * s = PyObject_Str (value);
		*this, "'", PyString_AsString (s), "'";
		decref (s);
	}

	if (trace)
	{
		if (type || value)
			fputs ("\nTraceback:\n", f);
		PyTraceBack_Print (trace, 0);
	}

	xdecref (type);
	xdecref (value);
	xdecref (trace);

	return *this;
}

/* ////////////////////////////////////////////// */

static PyObject *
_post_call (Py<Object> * p, char * method, PyObject * o)
{
	if (o)
	{
		/* print return value */
		if (o != Py_None)
		{
			message m;
			m, p, "::", method, "(): ";
			PyObject_Print (o, message::f, 0);
		}
	}
	else
	{
		/* it blew it */
		message m;
		m, p, "::", method, "(): ";
		m.py_err();
	}

	return o;
}

PyObject *
Py<Object>::call (char * name, char * format, ...)
{
	PyObject * method = PyObject_GetAttrString (this, name);

	if (method == 0)
		return 0;
					  
	va_list vargs;

	va_start (vargs, format);
	PyObject * args = Py_VaBuildValue (format, vargs);
	va_end (vargs);

	if (args == 0)
	{
		decref (method);
		return _post_call (this, name, 0);
	}

	PyObject * result = PyEval_CallObject (method, args);
	
	decref (method);
	decref (args);

	return _post_call (this, name, result);
}

/* ////////////////////////////////////////////// */

static void
_count (message & m, int n, char * item)
{
	m, n, " ", item, n == 1 ? "" : "s", " ";
}

/* only do cleaning up when the last Module is destroyed */
static int _instances;

class Module
: public Object
{
	public:
		Module()
			{ ++_instances; }

		~Module()
			{
				if (--_instances > 0)
					return;

				{
					message m;

					_count (m, Object::live_objects - 1, "object");
					_count (m, Packet::live_packets, "packet");
				}

				fclose (message::f);
			}
};

/* //////////////////////////////////////// */

#include "milk.thunk.h"
#if PYTHUNK
	Class (Object, None)
	Class (Module, Object)
#endif

extern "C" 
void initmilk()
{
	dlopen (0, RTLD_LAZY | RTLD_GLOBAL);
	
	PyEval_InitThreads();
	pthread_key_create (&Thread::key, 0);

	message::init ("stdmsg");
	message(), "\n\n\n// This, too, shall pass /////////////////////////////////";
	
	PyObject * self = Py_InitModule4 
		(
			"milk", 
			0, /* module methods */
			"http://quitte.de/milk.html",
			(PyObject *) 0,
			PYTHON_API_VERSION
		);
	PyObject * dict = PyModule_GetDict (self);
	
	new PyType<Object> ("Object", dict);
	new PyType<Module> ("__Module", dict);

	PyObject * o = new Py<Module>();
	PyDict_SetItemString (dict, "__module", o);
	decref (o);

	init_Flow (dict);
	init_RXTX (dict);
	init_Plugin (dict);
	init_MIDI (dict);
	init_Event (dict);
	init_ALSA (dict);
	init_Jack (dict);
}
